<?php
/*
Plugin Name: Community Cloud
Plugin URI: http://www.webaddict.co.za/2007/09/04/community-cloud-wordpress-plugin
Description: This plugin searches your wordpress database and displays a 'tag cloud' of all the people in your community who have contributed to our blog by commenting.  The more comments someone has made on your blog the bigger and bolder their tag.  You also have options to customize the community cloud to your needs.
People can see how big your community is in a more visually impactful way, as well as being able to see who the major contributors are compared to the people who just comment once off.
Author: Miguel dos Santos, (Modifications: Michael Champanis <michael.champanis@gmail.com>)
Author URI: http://www.webaddict.co.za
Version: 2.0
*/

require_once('includes/common.php');

// Function: Activate / Deactivate Module 
function community_cloud_activate() {
  global $wpdb, $current_user;
	//need to redclare thes because of global var issue when plugin is activated (http://wordpress.org/support/topic/201309)
	$community_cloud_table_name = $wpdb->prefix."community_cloud";
	
	if($wpdb->get_var("SHOW TABLES LIKE '".$community_cloud_table_name."'") != $community_cloud_table_name) {
		
		// drop and recreate the table
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		
		$sql = "CREATE TABLE IF NOT EXISTS `".$community_cloud_table_name."` (
	  `id` int(11) NOT NULL AUTO_INCREMENT,	
	  `name` varchar(255) NOT NULL,
		`email` varchar(255) NOT NULL,
	  `url` varchar(255) NOT NULL,
		`last_comment_date` int(11) NOT NULL,
	  `count` int(5) DEFAULT '0' NOT NULL,
	  `hcard_url` varchar(255) NOT NULL,
		PRIMARY KEY (`id`)
		);";
		dbDelta($sql);
		
		add_option("community_cloud_db_version", '1.0');
		add_option("cc_position", __('0', 'cc'));
		
		// Lets add some default options
		$a = '';
		$authors = $wpdb->get_results("SELECT user_email from $wpdb->users ORDER BY display_name");
		foreach($authors as $i) $a .= $i->user_email.' , ';
		add_option('cc_exclude_email', __(rtrim($a,','), 'cc'));
		add_option('cc_exclude_url', __(get_option('siteurl'), 'cc'));
		add_option('cc_empty_urls', __('0', 'cc'));
		add_option('cc_threshold', __('1', 'cc'));
		add_option('cc_maxtagsize', __('500', 'cc'));
		add_option('cc_mintagsize', __('90', 'cc'));
		add_option('cc_fontunits', __('%', 'cc'));
		add_option('cc_maxtagcolour', __('#1111AA', 'cc'));
		add_option('cc_mintagcolour', __('#3377CC', 'cc'));
		add_option('cc_show_comment_count', __('tooltip', 'cc'));
		add_option('cc_seperator', __('.', 'cc'));
		add_option('cc_order_by', __('random', 'cc'));
		// social options
		add_option('cc_creator_credit', __('1', 'cc'));
		add_option('cc_author_name', __($current_user->display_name, 'cc'));
		add_option('cc_social_friendfeed', __('', 'cc'));
		add_option('cc_social_twitter', __('', 'cc'));
		add_option('cc_social_facebook', __('', 'cc'));
		add_option('cc_advanced_hcard', __('0', 'cc'));
		
		if($write_flag = is_writable(ABSPATH)) $write_path = ABSPATH.'/hcard-social.html';
		elseif($write_flag = is_writable(ABSPATH.$community_cloud_path.'/services')) $write_path = ABSPATH.$community_cloud_path.'/services/hcard-social.html';
		if($write_flag) {
			if ($f = fopen($write_path, 'w+')) {
				$blog_url = get_option('siteurl');
				$hcard =<<<HCARD
<div id="hcard-$current_user->display_name" class="vcard">
<span class="fn n">
 <span class="given-name">$current_user->display_name</span>
</span>  
<span class="url">
 <a class="blog" href="$blog_url">$blog_url</a>
 <a class="friendfeed" href=""></a>
 <a class="twitter" href=""></a>
 <a class="facebook" href=""></a>   
</span>
</div>
HCARD;
				fwrite($f, $hcard);
				fclose($f);
			}
		}
		add_option('cc_social_hcard', __($hcard, 'cc'));
		
		//prepopulate the community cloud table
		community_cloud_build_cloud();
	}
}
register_activation_hook(__FILE__, 'community_cloud_activate');

function community_cloud_deactivate() {
	global $wpdb, $community_cloud_table_name;
	// drop table
	$wpdb->query( "DROP TABLE IF EXISTS ".$community_cloud_table_name.";" );
	// delete options
	delete_option('cc_position');
	delete_option('cc_exclude_email');
	delete_option('cc_exclude_url');
	delete_option('cc_empty_urls');
	delete_option('cc_threshold');
	delete_option('cc_maxtagsize');
	delete_option('cc_mintagsize');
	delete_option('cc_fontunits');
	delete_option('cc_maxtagcolour');
	delete_option('cc_mintagcolour');
	delete_option('cc_show_comment_count');
	delete_option('cc_seperator');
	delete_option('cc_order_by');
	delete_option('cc_creator_credit');
	delete_option('cc_author_name');
	delete_option('cc_social_friendfeed');
	delete_option('cc_social_twitter');
	delete_option('cc_social_facebook');
	delete_option('cc_advanced_hcard');
	delete_option('cc_social_hcard');
}
register_deactivation_hook(__FILE__, 'community_cloud_deactivate');

// Function: Page Navigation Option Menu
function community_cloud_menu() {
	global $community_cloud_plugin_basepath;
	if (function_exists('add_options_page')) {
		add_options_page(__('Community Cloud', 'cc'), __('Community Cloud', 'cc'), 'manage_options', $community_cloud_plugin_basepath.'/community_cloud-options.php');
	}
}
add_action('admin_menu', 'community_cloud_menu');

//update community cloud when user posts comment  and/or comment status changes
function community_cloud_update($comment_id, $status = 1) {
	global $wpdb, $community_cloud_table_name, $community_cloud_remote_path;
	//get comment info
	$comment = $wpdb->get_row("SELECT * FROM ".$wpdb->comments." WHERE comment_ID = ".$comment_id, ARRAY_A);
	//if comment approved, add or update community cloud table
	if($status==1 || $status='approved') {
		$hcard = '';
		$comment['comment_author_url'] = ltrim($comment['comment_author_url'], '/');
		if($link_flag = community_cloud_valid_url($comment['comment_author_url'])) {
			$temp_author_url = parse_url($author_url);
			$real_author_url = $temp_author_url['scheme'].'://'.$temp_author_url['host'];
			if (community_cloud_url_exists($real_author_url."/hcard-social.html")) $hcard = $real_author_url."/hcard-social.html";
			elseif (community_cloud_url_exists($real_author_url.$community_cloud_remote_path."/hcard-social.html")) $hcard = $real_author_url.$community_cloud_remote_path."/hcard-social.html";
		}
		//try update 
		$wpdb->query("UPDATE ".$community_cloud_table_name." SET count = count + 1, last_comment_date = ".strtotime($comment['comment_date']).", url = '".$comment['comment_author_url']."', hcard = '".$status."' WHERE email = '".$comment['comment_author_email']."';");
		if(!$wpdb->last_results) {
			// if can't update add new, only if comment count > threshold
			$count_result = $wpdb->get_results("SELECT count(*) as counter FROM ".$wpdb->comments." as c WHERE c.comment_author = '".$comment['comment_author']."' AND c.comment_approved = '1';");
			if($count_result[0]->counter >= get_option('cc_threshold')) {
				$wpdb->query("INSERT INTO ".$community_cloud_table_name." (name, email, url, last_comment_date, count, hcard_url) VALUES ('".$comment['comment_author']."', '".$comment['comment_author_email']."', '".$comment['comment_author_url']."', ".strtotime($comment['comment_date']).", 1, '".$status."');");
			}
		}
	} elseif($status=='hold' || $status='delete') {
		// if count falls below threshold, remove it from the table
		$count_result = $wpdb->get_results("SELECT count(*) as counter FROM ".$wpdb->comments." as c WHERE c.comment_author_email = '".$comment['comment_author_email']."' AND c.comment_approved = '1';");
		if($count_result[0]->counter <= get_option('cc_threshold')) {
			$wpdb->query("DELETE FROM ".$community_cloud_table_name." WHERE email = '".$comment['comment_author_email']."';");
		} else {
			// else update coment count minus 1
			$wpdb->query("UPDATE ".$community_cloud_table_name." SET count = count - 1 WHERE email = '".$comment['comment_author_email']."';");
		}
	}
}
add_action('comment_post', 'community_cloud_update');
add_action('wp_set_comment_status', 'community_cloud_update');

//readjust community cloud after manually editing comments
function community_cloud_rebuild($comment_id) {
	//the comment that was edited in the last 10 seconds
	community_cloud_build_cloud($comment_id);
}
add_action('edit_comment', 'community_cloud_rebuild');

//readjust community cloud table if we delete a comment
function community_cloud_delete_comment($comment_id) {
	//the comment that was edited in the last
	community_cloud_update($comment_id,'delete');
}
add_action('delete_comment', 'community_cloud_delete_comment'); 


// Wrapper function which calls the form.
function community_cloud_display($content , $count=0) {
	if(substr_count($content, '<!--community cloud-->')) {
		global $wpdb, $community_cloud_table_name, $community_cloud_path;
		$threshold = get_option('cc_threshold');
		$cloud = '';
		
		$request = "SELECT * FROM ".$community_cloud_table_name." ORDER BY ";
		switch(get_option('cc_order_by')) {
			case 'recent':
				$request .= "last_comment_date DESC";
				break;
			case 'top':
				$request .= "count DESC";
				break;
			default:
				$request .= "RAND()";
		}
		if($count!=0) $request .= " LIMIT 0,".$count;
		
		$comments = $wpdb->get_results($request);
		foreach ($comments as $comment) {
			$extra_tags = $extra_img = '';
			if($link_flag = community_cloud_valid_url($comment->url)) {
				if (isset($comment->hcard_url) && trim($comment->hcard_url)!='' && community_cloud_hcard_exists($comment->hcard_url)) {
					$extra_img = '<img src="http://www.google.com/s2/favicons?domain='.substr_replace($comment->url,'',0,strpos($comment->url,'/')+2).'" alt="'.$comment->name.'" /> ';
					$extra_tags = " class='friend-info' rel='".$community_cloud_path."/popup.php?hcard=".urlencode($comment->hcard_url)."'";
				} else {
					//update community cloud table that hcard for this user is broken, or doesn't exist, so don't check in future
					$wpdb->query('UPDATE '.$community_cloud_table_name.' SET hcard_url = "" WHERE id = "'.$comment->id.'"');
				}
			}
			
			if(get_option('cc_empty_urls') || $link_flag) {
				$cloud .= ' <nobr>';
				$comments_tag = ($comment->count == 1) ? 'comment' : 'comments';
				if($link_flag) $cloud .= " <a".$extra_tags." href='".$comment->url."' title='".$comment->name.((get_option('cc_show_comment_count')=='tooltip')?(" (".$comment->count." ".$comments_tag.")"):"")."' style='font-size:".community_cloud_GetFontSizeForWeight($comment->count).";".community_cloud_GetColorForWeight($comment->count).";'>".$extra_img;
				else $cloud .= " <span style='font-size:".community_cloud_GetFontSizeForWeight($comment->count/$threshold).";".community_cloud_GetColorForWeight($comment->count/$threshold).";'>";
				$cloud .= $comment->name.((get_option('cc_show_comment_count')=='link')?(" (".$comment->count.")"):"");
				if($link_flag) $cloud .= "</a> ";
				else $cloud .= "</span> ";
				$cloud .= '</nobr> ';
				$cloud .= get_option('cc_seperator');
			}
		}
		
		if(get_option('cc_creator_credit')==1) $cloud .= "<div align='right'>
		<a href='http://www.webaddict.co.za/community-cloud' title='Get the community cloud for your own wordpress blog'>
			<img src='".get_option('siteurl').$community_cloud_path."/images/sharecloud.png' alt='get the community cloud plugin for your blog' />
		</a></div>";
		return str_replace('<!--community cloud-->', $cloud, $content);
	} else {
		return $content;
	}
}
add_filter('the_content', 'community_cloud_display');

// This gets called at the plugins_loaded action
function widget_community_cloud_init() {
	
	// Check for the required API functions
	if ( !function_exists('register_sidebar_widget') || !function_exists('register_widget_control') )
		return;

	// This saves options and prints the widget's config form.
	function widget_community_cloud_control() {
		$options = $newoptions = get_option('widget_community_cloud');
		if ( $_POST['community_cloud-submit'] ) {
			$newoptions['title'] = strip_tags(stripslashes($_POST['community_cloud-title']));
			$newoptions['count'] = (int) $_POST['community_cloud-count'];
		}
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_community_cloud', $options);
		}
	?>
		<div style="text-align:right">
		<label for="community_cloud-title" style="line-height:35px;display:block;"><?php _e('Widget Title:', 'widgets'); ?> <input type="text" id="community_cloud-title" name="community_cloud-title" value="<?php echo wp_specialchars($options['title'], true); ?>" /></label>
		<label for="community_cloud-count" style="line-height:35px;display:block;"><?php _e('Number of Links in Cloud:', 'widgets'); ?>
		<select id="community_cloud-count" name="community_cloud-count">
			<option value="0"<?php if(0==$options['count']) echo "selected=selected"; ?>>Show All</option>
			<option value="100"<?php if(100==$options['count']) echo "selected=selected"; ?>>100</option>
			<option value="50"<?php if(50==$options['count']) echo "selected=selected"; ?>>50</option>
			<option value="25"<?php if(25==$options['count']) echo "selected=selected"; ?>>25</option>
			<option value="10"<?php if(10==$options['count']) echo "selected=selected"; ?>>10</option>
		</select></label>
		<input type="hidden" name="community_cloud-submit" id="community_cloud-submit" value="1" />
		</div>
	<?php
	}

	// This prints the widget
	function widget_community_cloud($args) {
		extract($args);
		$defaults = array('count' => 0, 'title' => 'Community Cloud');
		$options = (array) get_option('widget_community_cloud');

		foreach ( $defaults as $key => $value ) {
			if ( !isset($options[$key]) ) $options[$key] = $defaults[$key];
		}
		echo $before_widget;
		echo $before_title . $options['title'] . $after_title; 
		echo community_cloud_display( "<!--community cloud-->",  $options['count']);
		echo $after_widget; 
	}

	// Tell Dynamic Sidebar about our new widget and its control
	register_sidebar_widget(array('Community Cloud', 'widgets'), 'widget_community_cloud');
	register_widget_control(array('Community Cloud', 'widgets'), 'widget_community_cloud_control');
}
add_action('widgets_init', 'widget_community_cloud_init');

// add the plugin javascript files (but don't duplicate existing scripts)
function community_cloud_add_scripts() {
	global $community_cloud_path;
  wp_register_script('jquery-hover-intent', $community_cloud_path.'/js/jquery.hoverIntent.js', false, '');
  wp_register_script('jquery-cluetip', $community_cloud_path.'/js/jquery.cluetip.js', false, '0.9.8');
  wp_register_script('behaviours', $community_cloud_path.'/js/behaviours.js');  
  
  wp_enqueue_script('jquery');
  wp_enqueue_script('jquery-hover-intent');
  wp_enqueue_script('jquery-cluetip');
  wp_enqueue_script('behaviours');
}
add_filter('wp_print_scripts', 'community_cloud_add_scripts');

// print css files to html head
function community_cloud_print_styles() {
	global $community_cloud_path;
	print "\n".'<link rel="stylesheet" href="'.get_option('siteurl').$community_cloud_path.'/css/jquery.cluetip.css" type="text/css" />'.
				"\n".'<link rel="stylesheet" href="'.get_option('siteurl').$community_cloud_path.'/css/popup.css" type="text/css" />'."\n";
}
add_action('wp_head', 'community_cloud_print_styles');

?>